@@ -1,14 +1,12 @@
-//! 2D graphics/rendering. Fonts, textures, color, geometry, tessellation etc.
+//! A simple 2D graphics library for turning simple 2D shapes and text into textured triangles.
+//!
+//! Made for [`egui`](https://github.com/emilk/egui/).
+//!
+//! Create some [`Shape`]:s and pass them to [`tessellate_shapes`] to generate [`Mesh`]:es
+//! that you can then paint using some graphics API of your choice (e.g. OpenGL).
 
 // Forbid warnings in release builds:
 #![cfg_attr(not(debug_assertions), deny(warnings))]
-// Disabled so we can support rust 1.51:
-// #![deny(
-//     rustdoc::broken_intra_doc_links,
-//     rustdoc::invalid_codeblock_attributes,
-//     rustdoc::missing_crate_level_docs,
-//     rustdoc::private_intra_doc_links
-// )]
 #![forbid(unsafe_code)]
 #![warn(
     clippy::all,
@@ -17,6 +15,7 @@
     clippy::checked_conversions,
     clippy::dbg_macro,
     clippy::debug_assert_with_mut_call,
+    clippy::disallowed_method,
     clippy::doc_markdown,
     clippy::empty_enum,
     clippy::enum_glob_use,
@@ -26,12 +25,17 @@
     clippy::explicit_into_iter_loop,
     clippy::fallible_impl_from,
     clippy::filter_map_next,
+    clippy::flat_map_option,
     clippy::float_cmp_const,
     clippy::fn_params_excessive_bools,
+    clippy::from_iter_instead_of_collect,
     clippy::if_let_mutex,
+    clippy::implicit_clone,
     clippy::imprecise_flops,
     clippy::inefficient_to_string,
     clippy::invalid_upcast_comparisons,
+    clippy::large_digit_groups,
+    clippy::large_stack_arrays,
     clippy::large_types_passed_by_value,
     clippy::let_unit_value,
     clippy::linkedlist,
@@ -40,8 +44,10 @@
     clippy::manual_ok_or,
     clippy::map_err_ignore,
     clippy::map_flatten,
+    clippy::map_unwrap_or,
     clippy::match_on_vec_items,
     clippy::match_same_arms,
+    clippy::match_wild_err_arm,
     clippy::match_wildcard_for_single_variants,
     clippy::mem_forget,
     clippy::mismatched_target_os,
@@ -51,14 +57,16 @@
     clippy::mutex_integer,
     clippy::needless_borrow,
     clippy::needless_continue,
+    clippy::needless_for_each,
     clippy::needless_pass_by_value,
     clippy::option_option,
     clippy::path_buf_push_overwrite,
     clippy::ptr_as_ptr,
-    clippy::pub_enum_variant_names,
     clippy::ref_option_ref,
     clippy::rest_pat_in_fully_bound_structs,
     clippy::same_functions_in_if_condition,
+    clippy::semicolon_if_nothing_returned,
+    clippy::single_match_else,
     clippy::string_add_assign,
     clippy::string_add,
     clippy::string_lit_as_bytes,
@@ -70,9 +78,9 @@
     clippy::unused_self,
     clippy::useless_transmute,
     clippy::verbose_file_reads,
-    clippy::wrong_pub_self_convention,
     clippy::zero_sized_map_values,
     future_incompatible,
+    missing_crate_level_docs,
     nonstandard_style,
     rust_2018_idioms
 )]
@@ -90,19 +98,22 @@ mod stroke;
 pub mod tessellator;
 pub mod text;
 mod texture_atlas;
+pub mod util;
 
 pub use {
     color::{Color32, Rgba},
     mesh::{Mesh, Mesh16, Vertex},
     shadow::Shadow,
-    shape::Shape,
+    shape::{CircleShape, PathShape, RectShape, Shape, TextShape},
     stats::PaintStats,
     stroke::Stroke,
-    tessellator::{TessellationOptions, Tessellator},
-    text::{Galley, TextStyle},
+    tessellator::{tessellate_shapes, TessellationOptions, Tessellator},
+    text::{Fonts, Galley, TextStyle},
     texture_atlas::{Texture, TextureAtlas},
 };
 
+pub use emath::{pos2, vec2, Pos2, Rect, Vec2};
+
 pub use ahash;
 pub use emath;
 
@@ -114,6 +125,7 @@ pub const WHITE_UV: emath::Pos2 = emath::pos2(0.0, 0.0);
 
 /// What texture to use in a [`Mesh`] mesh.
 #[derive(Clone, Copy, Debug, PartialEq, Eq, Hash)]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
 pub enum TextureId {
     /// The egui font texture.
     /// If you don't want to use a texture, pick this and the [`WHITE_UV`] for uv-coord.
@@ -130,18 +142,10 @@ impl Default for TextureId {
     }
 }
 
-pub(crate) struct PaintRect {
-    pub rect: emath::Rect,
-    /// How rounded the corners are. Use `0.0` for no rounding.
-    pub corner_radius: f32,
-    pub fill: Color32,
-    pub stroke: Stroke,
-}
-
 /// A [`Shape`] within a clip rectangle.
 ///
 /// Everything is using logical points.
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, PartialEq)]
 pub struct ClippedShape(
     /// Clip / scissor rectangle.
     /// Only show the part of the [`Shape`] that falls within this.
@@ -154,6 +158,7 @@ pub struct ClippedShape(
 ///
 /// Everything is using logical points.
 #[derive(Clone, Debug)]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
 pub struct ClippedMesh(
     /// Clip / scissor rectangle.
     /// Only show the part of the [`Mesh`] that falls within this.
@@ -164,11 +169,11 @@ pub struct ClippedMesh(
 
 // ----------------------------------------------------------------------------
 
-/// An assert that is only active when `egui` is compiled with the `egui_assert` feature
-/// or with the `debug_egui_assert` feature in debug builds.
+/// An assert that is only active when `epaint` is compiled with the `extra_asserts` feature
+/// or with the `extra_debug_asserts` feature in debug builds.
 #[macro_export]
 macro_rules! epaint_assert {
-    ($($arg:tt)*) => {
+    ($($arg: tt)*) => {
         if cfg!(any(
             feature = "extra_asserts",
             all(feature = "extra_debug_asserts", debug_assertions),
@@ -177,3 +182,17 @@ macro_rules! epaint_assert {
         }
     }
 }
+
+// ----------------------------------------------------------------------------
+
+#[inline(always)]
+pub(crate) fn f32_hash<H: std::hash::Hasher>(state: &mut H, f: f32) {
+    if f == 0.0 {
+        state.write_u8(0);
+    } else if f.is_nan() {
+        state.write_u8(1);
+    } else {
+        use std::hash::Hash;
+        f.to_bits().hash(state);
+    }
+}
