@@ -12,7 +12,7 @@ use std::f32::consts::TAU;
 // ----------------------------------------------------------------------------
 
 #[derive(Clone, Debug, Default)]
-pub struct PathPoint {
+struct PathPoint {
     pos: Pos2,
 
     /// For filled paths the normal is used for anti-aliasing (both strokes and filled areas).
@@ -31,7 +31,7 @@ pub struct PathPoint {
 /// to either to a stroke (with thickness) or a filled convex area.
 /// Used as a scratch-pad during tessellation.
 #[derive(Clone, Debug, Default)]
-struct Path(Vec<PathPoint>);
+pub struct Path(Vec<PathPoint>);
 
 impl Path {
     #[inline(always)]
@@ -41,7 +41,7 @@ impl Path {
 
     #[inline(always)]
     pub fn reserve(&mut self, additional: usize) {
-        self.0.reserve(additional)
+        self.0.reserve(additional);
     }
 
     #[inline(always)]
@@ -81,7 +81,7 @@ impl Path {
             for i in 1..n - 1 {
                 let mut n1 = (points[i + 1] - points[i]).normalized().rot90();
 
-                // Handle duplicated points (but not triplicated...):
+                // Handle duplicated points (but not triplicated…):
                 if n0 == Vec2::ZERO {
                     n0 = n1;
                 } else if n1 == Vec2::ZERO {
@@ -124,7 +124,7 @@ impl Path {
             let next_i = if i + 1 == n { 0 } else { i + 1 };
             let mut n1 = (points[next_i] - points[i]).normalized().rot90();
 
-            // Handle duplicated points (but not triplicated...):
+            // Handle duplicated points (but not triplicated…):
             if n0 == Vec2::ZERO {
                 n0 = n1;
             } else if n1 == Vec2::ZERO {
@@ -150,6 +150,31 @@ impl Path {
             n0 = n1;
         }
     }
+
+    /// Open-ended.
+    pub fn stroke_open(&self, stroke: Stroke, options: &TessellationOptions, out: &mut Mesh) {
+        stroke_path(&self.0, PathType::Open, stroke, options, out);
+    }
+
+    /// A closed path (returning to the first point).
+    pub fn stroke_closed(&self, stroke: Stroke, options: &TessellationOptions, out: &mut Mesh) {
+        stroke_path(&self.0, PathType::Closed, stroke, options, out);
+    }
+
+    pub fn stroke(
+        &self,
+        path_type: PathType,
+        stroke: Stroke,
+        options: &TessellationOptions,
+        out: &mut Mesh,
+    ) {
+        stroke_path(&self.0, path_type, stroke, options, out);
+    }
+
+    /// The path is taken to be closed (i.e. returning to the start again).
+    pub fn fill(&self, color: Color32, options: &TessellationOptions, out: &mut Mesh) {
+        fill_closed_path(&self.0, color, options, out);
+    }
 }
 
 pub mod path {
@@ -226,27 +251,36 @@ pub enum PathType {
     Open,
     Closed,
 }
-use self::PathType::{Closed, Open};
 
 /// Tessellation quality options
 #[derive(Clone, Copy, Debug, PartialEq)]
-#[cfg_attr(feature = "persistence", derive(serde::Deserialize, serde::Serialize))]
-#[cfg_attr(feature = "persistence", serde(default))]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
+#[cfg_attr(feature = "serde", serde(default))]
 pub struct TessellationOptions {
-    /// Size of a point in pixels, e.g. 2.0. Used to snap text to pixel boundaries.
+    /// Size of a point in pixels (DPI scaling), e.g. 2.0. Used to snap text to pixel boundaries.
     pub pixels_per_point: f32,
-    /// Size of a pixel in points, e.g. 0.5, or larger if you want more blurry edges.
+
+    /// The size of a pixel (in points), used for anti-aliasing (smoothing of edges).
+    /// This is normally the inverse of [`Self::pixels_per_point`],
+    /// but you can make it larger if you want more blurry edges.
     pub aa_size: f32,
+
     /// Anti-aliasing makes shapes appear smoother, but requires more triangles and is therefore slower.
-    /// By default this is enabled in release builds and disabled in debug builds.
+    /// This setting does not affect text.
+    /// Default: `true`.
     pub anti_alias: bool,
-    /// If `true` (default) cull certain primitives before tessellating them
+
+    /// If `true` (default) cull certain primitives before tessellating them.
+    /// This likely makes
     pub coarse_tessellation_culling: bool,
-    /// Output the clip rectangles to be painted?
+
+    /// Output the clip rectangles to be painted.
     pub debug_paint_clip_rects: bool,
-    /// Output the text-containing rectangles
+
+    /// Output the text-containing rectangles.
     pub debug_paint_text_rects: bool,
-    /// If true, no clipping will be done
+
+    /// If true, no clipping will be done.
     pub debug_ignore_clip_rects: bool,
 }
 
@@ -264,6 +298,16 @@ impl Default for TessellationOptions {
     }
 }
 
+impl TessellationOptions {
+    pub fn from_pixels_per_point(pixels_per_point: f32) -> Self {
+        Self {
+            pixels_per_point,
+            aa_size: 1.0 / pixels_per_point,
+            ..Default::default()
+        }
+    }
+}
+
 impl TessellationOptions {
     #[inline(always)]
     pub fn round_to_pixel(&self, point: f32) -> f32 {
@@ -275,7 +319,7 @@ impl TessellationOptions {
 fn fill_closed_path(
     path: &[PathPoint],
     color: Color32,
-    options: TessellationOptions,
+    options: &TessellationOptions,
     out: &mut Mesh,
 ) {
     if color == Color32::TRANSPARENT {
@@ -321,7 +365,7 @@ fn stroke_path(
     path: &[PathPoint],
     path_type: PathType,
     stroke: Stroke,
-    options: TessellationOptions,
+    options: &TessellationOptions,
     out: &mut Mesh,
 ) {
     if stroke.width <= 0.0 || stroke.color == Color32::TRANSPARENT {
@@ -419,7 +463,11 @@ fn stroke_path(
         out.reserve_triangles(2 * n as usize);
         out.reserve_vertices(2 * n as usize);
 
-        let last_index = if path_type == Closed { n } else { n - 1 };
+        let last_index = if path_type == PathType::Closed {
+            n
+        } else {
+            n - 1
+        };
         for i in 0..last_index {
             out.add_triangle(
                 idx + (2 * i + 0) % (2 * n),
@@ -464,7 +512,11 @@ fn mul_color(color: Color32, factor: f32) -> Color32 {
 
 // ----------------------------------------------------------------------------
 
-/// Converts [`Shape`]s into [`Mesh`].
+/// Converts [`Shape`]s into triangles ([`Mesh`]).
+///
+/// For performance reasons it is smart to reuse the same `Tessellator`.
+///
+/// Se also [`tessellate_shapes`], a convenient wrapper around [`Tessellator`].
 pub struct Tessellator {
     options: TessellationOptions,
     /// Only used for culling
@@ -474,6 +526,7 @@ pub struct Tessellator {
 }
 
 impl Tessellator {
+    /// Create a new [`Tessellator`].
     pub fn from_options(options: TessellationOptions) -> Self {
         Self {
             options,
@@ -485,29 +538,26 @@ impl Tessellator {
 
     /// Tessellate a single [`Shape`] into a [`Mesh`].
     ///
-    /// * `shape`: the shape to tessellate
-    /// * `options`: tessellation quality
-    /// * `tex_size`: size of the font texture (required to normalize glyph uv rectangles)
-    /// * `out`: where the triangles are put
-    /// * `scratchpad_path`: if you plan to run `tessellate_shape`
-    ///    many times, pass it a reference to the same `Path` to avoid excessive allocations.
+    /// * `tex_size`: size of the font texture (required to normalize glyph uv rectangles).
+    /// * `shape`: the shape to tessellate.
+    /// * `out`: triangles are appended to this.
     pub fn tessellate_shape(&mut self, tex_size: [usize; 2], shape: Shape, out: &mut Mesh) {
         let clip_rect = self.clip_rect;
-        let options = self.options;
+        let options = &self.options;
 
         match shape {
             Shape::Noop => {}
             Shape::Vec(vec) => {
                 for shape in vec {
-                    self.tessellate_shape(tex_size, shape, out)
+                    self.tessellate_shape(tex_size, shape, out);
                 }
             }
-            Shape::Circle {
+            Shape::Circle(CircleShape {
                 center,
                 radius,
                 fill,
                 stroke,
-            } => {
+            }) => {
                 if radius <= 0.0 {
                     return;
                 }
@@ -518,89 +568,101 @@ impl Tessellator {
                     return;
                 }
 
-                let path = &mut self.scratchpad_path;
-                path.clear();
-                path.add_circle(center, radius);
-                fill_closed_path(&path.0, fill, options, out);
-                stroke_path(&path.0, Closed, stroke, options, out);
+                self.scratchpad_path.clear();
+                self.scratchpad_path.add_circle(center, radius);
+                self.scratchpad_path.fill(fill, options, out);
+                self.scratchpad_path.stroke_closed(stroke, options, out);
             }
             Shape::Mesh(mesh) => {
-                if mesh.is_valid() {
-                    out.append(mesh);
-                } else {
+                if !mesh.is_valid() {
                     crate::epaint_assert!(false, "Invalid Mesh in Shape::Mesh");
+                    return;
                 }
+
+                if options.coarse_tessellation_culling && !clip_rect.intersects(mesh.calc_bounds())
+                {
+                    return;
+                }
+
+                out.append(mesh);
             }
             Shape::LineSegment { points, stroke } => {
-                let path = &mut self.scratchpad_path;
-                path.clear();
-                path.add_line_segment(points);
-                stroke_path(&path.0, Open, stroke, options, out);
-            }
-            Shape::Path {
-                points,
-                closed,
-                fill,
-                stroke,
-            } => {
-                if points.len() >= 2 {
-                    let path = &mut self.scratchpad_path;
-                    path.clear();
-                    if closed {
-                        path.add_line_loop(&points);
-                    } else {
-                        path.add_open_points(&points);
-                    }
-
-                    if fill != Color32::TRANSPARENT {
-                        crate::epaint_assert!(
-                            closed,
-                            "You asked to fill a path that is not closed. That makes no sense."
-                        );
-                        fill_closed_path(&path.0, fill, options, out);
-                    }
-                    let typ = if closed { Closed } else { Open };
-                    stroke_path(&path.0, typ, stroke, options, out);
+                if stroke.is_empty() {
+                    return;
                 }
+
+                if options.coarse_tessellation_culling
+                    && !clip_rect
+                        .intersects(Rect::from_two_pos(points[0], points[1]).expand(stroke.width))
+                {
+                    return;
+                }
+
+                self.scratchpad_path.clear();
+                self.scratchpad_path.add_line_segment(points);
+                self.scratchpad_path.stroke_open(stroke, options, out);
             }
-            Shape::Rect {
-                rect,
-                corner_radius,
-                fill,
-                stroke,
-            } => {
-                let rect = PaintRect {
-                    rect,
-                    corner_radius,
-                    fill,
-                    stroke,
-                };
-                self.tessellate_rect(&rect, out);
+            Shape::Path(path_shape) => {
+                self.tessellate_path(path_shape, out);
             }
-            Shape::Text {
-                pos,
-                galley,
-                color,
-                fake_italics,
-            } => {
+            Shape::Rect(rect_shape) => {
+                self.tessellate_rect(&rect_shape, out);
+            }
+            Shape::Text(text_shape) => {
                 if options.debug_paint_text_rects {
+                    let rect = text_shape.galley.rect.translate(text_shape.pos.to_vec2());
                     self.tessellate_rect(
-                        &PaintRect {
-                            rect: Rect::from_min_size(pos, galley.size).expand(0.5),
-                            corner_radius: 2.0,
-                            fill: Default::default(),
-                            stroke: (0.5, color).into(),
-                        },
+                        &RectShape::stroke(rect.expand(0.5), 2.0, (0.5, Color32::GREEN)),
                         out,
                     );
                 }
-                self.tessellate_text(tex_size, pos, &galley, color, fake_italics, out);
+                self.tessellate_text(tex_size, text_shape, out);
             }
         }
     }
 
-    pub(crate) fn tessellate_rect(&mut self, rect: &PaintRect, out: &mut Mesh) {
-        let PaintRect {
+    pub(crate) fn tessellate_path(&mut self, path_shape: PathShape, out: &mut Mesh) {
+        if path_shape.points.len() < 2 {
+            return;
+        }
+
+        if self.options.coarse_tessellation_culling
+            && !path_shape.bounding_rect().intersects(self.clip_rect)
+        {
+            return;
+        }
+
+        let PathShape {
+            points,
+            closed,
+            fill,
+            stroke,
+        } = path_shape;
+
+        self.scratchpad_path.clear();
+        if closed {
+            self.scratchpad_path.add_line_loop(&points);
+        } else {
+            self.scratchpad_path.add_open_points(&points);
+        }
+
+        if fill != Color32::TRANSPARENT {
+            crate::epaint_assert!(
+                closed,
+                "You asked to fill a path that is not closed. That makes no sense."
+            );
+            self.scratchpad_path.fill(fill, &self.options, out);
+        }
+        let typ = if closed {
+            PathType::Closed
+        } else {
+            PathType::Open
+        };
+        self.scratchpad_path.stroke(typ, stroke, &self.options, out);
+    }
+
+    pub(crate) fn tessellate_rect(&mut self, rect: &RectShape, out: &mut Mesh) {
+        let RectShape {
             mut rect,
             corner_radius,
             fill,
@@ -625,107 +687,99 @@ impl Tessellator {
         path.clear();
         path::rounded_rectangle(&mut self.scratchpad_points, rect, corner_radius);
         path.add_line_loop(&self.scratchpad_points);
-        fill_closed_path(&path.0, fill, self.options, out);
-        stroke_path(&path.0, Closed, stroke, self.options, out);
+        path.fill(fill, &self.options, out);
+        path.stroke_closed(stroke, &self.options, out);
     }
 
-    pub fn tessellate_text(
-        &mut self,
-        tex_size: [usize; 2],
-        pos: Pos2,
-        galley: &super::Galley,
-        color: Color32,
-        fake_italics: bool,
-        out: &mut Mesh,
-    ) {
-        if color == Color32::TRANSPARENT || galley.is_empty() {
+    pub fn tessellate_text(&mut self, tex_size: [usize; 2], text_shape: TextShape, out: &mut Mesh) {
+        let TextShape {
+            pos: galley_pos,
+            galley,
+            underline,
+            override_text_color,
+            angle,
+        } = text_shape;
+
+        if galley.is_empty() {
             return;
         }
-        if cfg!(any(
-            feature = "extra_asserts",
-            all(feature = "extra_debug_asserts", debug_assertions),
-        )) {
-            galley.sanity_check();
-        }
+
+        out.vertices.reserve(galley.num_vertices);
+        out.indices.reserve(galley.num_indices);
 
         // The contents of the galley is already snapped to pixel coordinates,
         // but we need to make sure the galley ends up on the start of a physical pixel:
-        let pos = pos2(
-            self.options.round_to_pixel(pos.x),
-            self.options.round_to_pixel(pos.y),
+        let galley_pos = pos2(
+            self.options.round_to_pixel(galley_pos.x),
+            self.options.round_to_pixel(galley_pos.y),
         );
 
-        let num_chars = galley.char_count_excluding_newlines();
-        out.reserve_triangles(num_chars * 2);
-        out.reserve_vertices(num_chars * 4);
+        let uv_normalizer = vec2(1.0 / tex_size[0] as f32, 1.0 / tex_size[1] as f32);
 
-        let inv_tex_w = 1.0 / tex_size[0] as f32;
-        let inv_tex_h = 1.0 / tex_size[1] as f32;
-
-        let clip_slack = 2.0; // Some fudge to handle letters that are slightly larger than expected.
-        let clip_rect_min_y = self.clip_rect.min.y - clip_slack;
-        let clip_rect_max_y = self.clip_rect.max.y + clip_slack;
+        let rotator = Rot2::from_angle(angle);
 
         for row in &galley.rows {
-            let row_min_y = pos.y + row.y_min;
-            let row_max_y = pos.y + row.y_max;
-            let is_line_visible = clip_rect_min_y <= row_max_y && row_min_y <= clip_rect_max_y;
+            if row.visuals.mesh.is_empty() {
+                continue;
+            }
 
-            if self.options.coarse_tessellation_culling && !is_line_visible {
+            let mut row_rect = row.visuals.mesh_bounds;
+            if angle != 0.0 {
+                row_rect = row_rect.rotate_bb(rotator);
+            }
+            row_rect = row_rect.translate(galley_pos.to_vec2());
+
+            if self.options.coarse_tessellation_culling && !self.clip_rect.intersects(row_rect) {
                 // culling individual lines of text is important, since a single `Shape::Text`
                 // can span hundreds of lines.
                 continue;
             }
 
-            for (x_offset, uv_rect) in row.x_offsets.iter().zip(&row.uv_rects) {
-                if let Some(glyph) = uv_rect {
-                    let mut left_top = pos + glyph.offset + vec2(*x_offset, row.y_min);
-                    left_top.x = self.options.round_to_pixel(left_top.x); // Pixel-perfection.
-                    left_top.y = self.options.round_to_pixel(left_top.y); // Pixel-perfection.
+            let index_offset = out.vertices.len() as u32;
 
-                    let rect = Rect::from_min_max(left_top, left_top + glyph.size);
-                    let uv = Rect::from_min_max(
-                        pos2(
-                            glyph.min.0 as f32 * inv_tex_w,
-                            glyph.min.1 as f32 * inv_tex_h,
-                        ),
-                        pos2(
-                            glyph.max.0 as f32 * inv_tex_w,
-                            glyph.max.1 as f32 * inv_tex_h,
-                        ),
-                    );
+            out.indices.extend(
+                row.visuals
+                    .mesh
+                    .indices
+                    .iter()
+                    .map(|index| index + index_offset),
+            );
 
-                    if fake_italics {
-                        let idx = out.vertices.len() as u32;
-                        out.add_triangle(idx, idx + 1, idx + 2);
-                        out.add_triangle(idx + 2, idx + 1, idx + 3);
+            out.vertices.extend(
+                row.visuals
+                    .mesh
+                    .vertices
+                    .iter()
+                    .enumerate()
+                    .map(|(i, vertex)| {
+                        let Vertex { pos, uv, mut color } = *vertex;
 
-                        let top_offset = rect.height() * 0.25 * Vec2::X;
+                        if let Some(override_text_color) = override_text_color {
+                            if row.visuals.glyph_vertex_range.contains(&i) {
+                                color = override_text_color;
+                            }
+                        }
 
-                        out.vertices.push(Vertex {
-                            pos: rect.left_top() + top_offset,
-                            uv: uv.left_top(),
+                        let offset = if angle == 0.0 {
+                            pos.to_vec2()
+                        } else {
+                            rotator * pos.to_vec2()
+                        };
+
+                        Vertex {
+                            pos: galley_pos + offset,
+                            uv: (uv.to_vec2() * uv_normalizer).to_pos2(),
                             color,
-                        });
-                        out.vertices.push(Vertex {
-                            pos: rect.right_top() + top_offset,
-                            uv: uv.right_top(),
-                            color,
-                        });
-                        out.vertices.push(Vertex {
-                            pos: rect.left_bottom(),
-                            uv: uv.left_bottom(),
-                            color,
-                        });
-                        out.vertices.push(Vertex {
-                            pos: rect.right_bottom(),
-                            uv: uv.right_bottom(),
-                            color,
-                        });
-                    } else {
-                        out.add_rect_with_uv(rect, uv, color);
-                    }
-                }
+                        }
+                    }),
+            );
+
+            if underline != Stroke::none() {
+                self.scratchpad_path.clear();
+                self.scratchpad_path
+                    .add_line_segment([row_rect.left_bottom(), row_rect.right_bottom()]);
+                self.scratchpad_path
+                    .stroke_open(underline, &self.options, out);
             }
         }
     }
@@ -733,13 +787,15 @@ impl Tessellator {
 
 /// Turns [`Shape`]:s into sets of triangles.
 ///
-/// The given shapes will be painted back-to-front (painters algorithm).
+/// The given shapes will tessellated in the same order as they are given.
 /// They will be batched together by clip rectangle.
 ///
-/// * `shapes`: the shape to tessellate
+/// * `shapes`: what to tessellate
 /// * `options`: tessellation quality
 /// * `tex_size`: size of the font texture (required to normalize glyph uv rectangles)
 ///
+/// The implementation uses a [`Tessellator`].
+///
 /// ## Returns
 /// A list of clip rectangles with matching [`Mesh`].
 pub fn tessellate_shapes(
@@ -775,14 +831,13 @@ pub fn tessellate_shapes(
             tessellator.clip_rect = Rect::EVERYTHING;
             tessellator.tessellate_shape(
                 tex_size,
-                Shape::Rect {
-                    rect: *clip_rect,
-                    corner_radius: 0.0,
-                    fill: Default::default(),
-                    stroke: Stroke::new(2.0, Color32::from_rgb(150, 255, 150)),
-                },
+                Shape::rect_stroke(
+                    *clip_rect,
+                    0.0,
+                    Stroke::new(2.0, Color32::from_rgb(150, 255, 150)),
+                ),
                 mesh,
-            )
+            );
         }
     }
 
