@@ -1,16 +1,10 @@
-use std::{
-    collections::BTreeMap,
-    hash::{Hash, Hasher},
-    sync::Arc,
-};
-
-use ahash::AHashMap;
+use std::{collections::BTreeMap, sync::Arc};
 
 use crate::{
     mutex::Mutex,
     text::{
         font::{Font, FontImpl},
-        Galley,
+        Galley, LayoutJob,
     },
     Texture, TextureAtlas,
 };
@@ -18,8 +12,8 @@ use crate::{
 // TODO: rename
 /// One of a few categories of styles of text, e.g. body, button or heading.
 #[derive(Copy, Clone, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
-#[cfg_attr(feature = "persistence", derive(serde::Deserialize, serde::Serialize))]
-#[cfg_attr(feature = "persistence", serde(rename_all = "snake_case"))]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
+#[cfg_attr(feature = "serde", serde(rename_all = "snake_case"))]
 pub enum TextStyle {
     /// Used when small text is needed.
     Small,
@@ -49,8 +43,8 @@ impl TextStyle {
 
 /// Which style of font: [`Monospace`][`FontFamily::Monospace`] or [`Proportional`][`FontFamily::Proportional`].
 #[derive(Copy, Clone, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
-#[cfg_attr(feature = "persistence", derive(serde::Deserialize, serde::Serialize))]
-#[cfg_attr(feature = "persistence", serde(rename_all = "snake_case"))]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
+#[cfg_attr(feature = "serde", serde(rename_all = "snake_case"))]
 pub enum FontFamily {
     /// A font where each character is the same width (`w` is the same width as `i`).
     Monospace,
@@ -61,12 +55,12 @@ pub enum FontFamily {
 /// The data of a `.ttf` or `.otf` file.
 pub type FontData = std::borrow::Cow<'static, [u8]>;
 
-fn rusttype_font_from_font_data(name: &str, data: &FontData) -> rusttype::Font<'static> {
+fn ab_glyph_font_from_font_data(name: &str, data: &FontData) -> ab_glyph::FontArc {
     match data {
-        std::borrow::Cow::Borrowed(bytes) => rusttype::Font::try_from_bytes(bytes),
-        std::borrow::Cow::Owned(bytes) => rusttype::Font::try_from_vec(bytes.clone()),
+        std::borrow::Cow::Borrowed(bytes) => ab_glyph::FontArc::try_from_slice(bytes),
+        std::borrow::Cow::Owned(bytes) => ab_glyph::FontArc::try_from_vec(bytes.clone()),
     }
-    .unwrap_or_else(|| panic!("Error parsing {:?} TTF/OTF font file", name))
+    .unwrap_or_else(|err| panic!("Error parsing {:?} TTF/OTF font file: {}", name, err))
 }
 
 /// Describes the font data and the sizes to use.
@@ -74,22 +68,52 @@ fn rusttype_font_from_font_data(name: &str, data: &FontData) -> rusttype::Font<'
 /// Often you would start with [`FontDefinitions::default()`] and then add/change the contents.
 ///
 /// ```
-/// let mut fonts = epaint::text::FontDefinitions::default();
+/// # use {epaint::text::{FontDefinitions, TextStyle, FontFamily}};
+/// # struct FakeEguiCtx {};
+/// # impl FakeEguiCtx { fn set_fonts(&self, _: FontDefinitions) {} }
+/// # let ctx = FakeEguiCtx {};
+/// let mut fonts = FontDefinitions::default();
+///
 /// // Large button text:
 /// fonts.family_and_size.insert(
-///     epaint::text::TextStyle::Button,
-///     (epaint::text::FontFamily::Proportional, 32.0));
+///     TextStyle::Button,
+///     (FontFamily::Proportional, 32.0)
+/// );
+///
+/// ctx.set_fonts(fonts);
+/// ```
+///
+/// You can also install your own custom fonts:
+/// ```
+/// # use {epaint::text::{FontDefinitions, TextStyle, FontFamily}};
+/// # struct FakeEguiCtx {};
+/// # impl FakeEguiCtx { fn set_fonts(&self, _: FontDefinitions) {} }
+/// # let ctx = FakeEguiCtx {};
+/// let mut fonts = FontDefinitions::default();
+///
+/// // Install my own font (maybe supporting non-latin characters):
+/// fonts.font_data.insert("my_font".to_owned(),
+///    std::borrow::Cow::Borrowed(include_bytes!("../../fonts/Ubuntu-Light.ttf"))); // .ttf and .otf supported
+///
+/// // Put my font first (highest priority):
+/// fonts.fonts_for_family.get_mut(&FontFamily::Proportional).unwrap()
+///     .insert(0, "my_font".to_owned());
+///
+/// // Put my font as last fallback for monospace:
+/// fonts.fonts_for_family.get_mut(&FontFamily::Monospace).unwrap()
+///     .push("my_font".to_owned());
+///
+/// ctx.set_fonts(fonts);
 /// ```
 #[derive(Clone, Debug, PartialEq)]
-#[cfg_attr(feature = "persistence", derive(serde::Deserialize, serde::Serialize))]
-#[cfg_attr(feature = "persistence", serde(default))]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
+#[cfg_attr(feature = "serde", serde(default))]
 pub struct FontDefinitions {
     /// List of font names and their definitions.
     /// The definition must be the contents of either a `.ttf` or `.otf` font file.
     ///
     /// `epaint` has built-in-default for these,
     /// but you can override them if you like.
-    #[cfg_attr(feature = "persistence", serde(skip))]
     pub font_data: BTreeMap<String, FontData>,
 
     /// Which fonts (names) to use for each [`FontFamily`].
@@ -113,12 +137,9 @@ impl Default for FontDefinitions {
 
         #[cfg(feature = "default_fonts")]
         {
-            // TODO: figure out a way to make the WASM smaller despite including fonts. Zip them?
-
-            // Use size 13 for this. NOTHING ELSE:
             font_data.insert(
-                "ProggyClean".to_owned(),
-                std::borrow::Cow::Borrowed(include_bytes!("../../fonts/ProggyClean.ttf")),
+                "Hack".to_owned(),
+                std::borrow::Cow::Borrowed(include_bytes!("../../fonts/Hack-Regular.ttf")),
             );
             font_data.insert(
                 "Ubuntu-Light".to_owned(),
@@ -139,7 +160,7 @@ impl Default for FontDefinitions {
             fonts_for_family.insert(
                 FontFamily::Monospace,
                 vec![
-                    "ProggyClean".to_owned(),
+                    "Hack".to_owned(),
                     "Ubuntu-Light".to_owned(), // fallback for √ etc
                     "NotoEmoji-Regular".to_owned(),
                     "emoji-icon-font".to_owned(),
@@ -164,9 +185,9 @@ impl Default for FontDefinitions {
         let mut family_and_size = BTreeMap::new();
         family_and_size.insert(TextStyle::Small, (FontFamily::Proportional, 10.0));
         family_and_size.insert(TextStyle::Body, (FontFamily::Proportional, 14.0));
-        family_and_size.insert(TextStyle::Button, (FontFamily::Proportional, 16.0));
+        family_and_size.insert(TextStyle::Button, (FontFamily::Proportional, 14.0));
         family_and_size.insert(TextStyle::Heading, (FontFamily::Proportional, 20.0));
-        family_and_size.insert(TextStyle::Monospace, (FontFamily::Monospace, 13.0)); // 13 for `ProggyClean`
+        family_and_size.insert(TextStyle::Monospace, (FontFamily::Monospace, 14.0));
 
         Self {
             font_data,
@@ -177,6 +198,8 @@ impl Default for FontDefinitions {
 }
 
 /// The collection of fonts used by `epaint`.
+///
+/// Required in order to paint text.
 pub struct Fonts {
     pixels_per_point: f32,
     definitions: FontDefinitions,
@@ -190,7 +213,9 @@ pub struct Fonts {
 }
 
 impl Fonts {
-    pub fn from_definitions(pixels_per_point: f32, definitions: FontDefinitions) -> Self {
+    /// Create a new [`Fonts`] for text layout.
+    /// This call is expensive, so only create on [`Fonts`] and then reuse it.
+    pub fn new(pixels_per_point: f32, definitions: FontDefinitions) -> Self {
         assert!(
             0.0 < pixels_per_point && pixels_per_point < 100.0,
             "pixels_per_point out of range: {}",
@@ -233,10 +258,7 @@ impl Fonts {
             let mut atlas = atlas.lock();
             let texture = atlas.texture_mut();
             // Make sure we seed the texture version with something unique based on the default characters:
-            use std::collections::hash_map::DefaultHasher;
-            let mut hasher = DefaultHasher::default();
-            texture.pixels.hash(&mut hasher);
-            texture.version = hasher.finish();
+            texture.version = crate::util::hash(&texture.pixels);
         }
 
         Self {
@@ -249,6 +271,11 @@ impl Fonts {
         }
     }
 
+    #[deprecated = "Renamed to Fonts::new"]
+    pub fn from_definitions(pixels_per_point: f32, definitions: FontDefinitions) -> Self {
+        Self::new(pixels_per_point, definitions)
+    }
+
     #[inline(always)]
     pub fn pixels_per_point(&self) -> f32 {
         self.pixels_per_point
@@ -263,6 +290,11 @@ impl Fonts {
         (point * self.pixels_per_point).round() / self.pixels_per_point
     }
 
+    #[inline(always)]
+    pub fn floor_to_pixel(&self, point: f32) -> f32 {
+        (point * self.pixels_per_point).floor() / self.pixels_per_point
+    }
+
     /// Call each frame to get the latest available font texture data.
     pub fn texture(&self) -> Arc<Texture> {
         let atlas = self.atlas.lock();
@@ -284,66 +316,58 @@ impl Fonts {
         self.fonts[&text_style].row_height()
     }
 
-    /// Always returns at least one row.
-    /// Will line break at `\n`.
-    pub fn layout_no_wrap(&self, text_style: TextStyle, text: String) -> Arc<Galley> {
-        self.layout_multiline(text_style, text, f32::INFINITY)
-    }
-
-    /// Typeset the given text onto one row.
-    /// Any `\n` will show up as the replacement character.
-    /// Always returns exactly one `Row` in the `Galley`.
+    /// Layout some text.
+    /// This is the most advanced layout function.
+    /// See also [`Self::layout`], [`Self::layout_no_wrap`] and
+    /// [`Self::layout_delayed_color`].
     ///
-    /// Most often you probably want `\n` to produce a new row,
-    /// and so [`Self::layout_no_wrap`] may be a better choice.
-    pub fn layout_single_line(&self, text_style: TextStyle, text: String) -> Arc<Galley> {
-        self.galley_cache.lock().layout(
-            &self.fonts,
-            LayoutJob {
-                text_style,
-                text,
-                layout_params: LayoutParams::SingleLine,
-            },
-        )
+    /// The implementation uses memoization so repeated calls are cheap.
+    pub fn layout_job(&self, job: LayoutJob) -> Arc<Galley> {
+        self.galley_cache.lock().layout(self, job)
     }
 
-    /// Always returns at least one row.
-    /// Will wrap text at the given width.
-    pub fn layout_multiline(
+    /// Will wrap text at the given width and line break at `\n`.
+    ///
+    /// The implementation uses memoization so repeated calls are cheap.
+    pub fn layout(
         &self,
-        text_style: TextStyle,
         text: String,
-        max_width_in_points: f32,
+        text_style: TextStyle,
+        color: crate::Color32,
+        wrap_width: f32,
     ) -> Arc<Galley> {
-        self.layout_multiline_with_indentation_and_max_width(
-            text_style,
+        let job = LayoutJob::simple(text, text_style, color, wrap_width);
+        self.layout_job(job)
+    }
+
+    /// Will line break at `\n`.
+    ///
+    /// The implementation uses memoization so repeated calls are cheap.
+    pub fn layout_no_wrap(
+        &self,
+        text: String,
+        text_style: TextStyle,
+        color: crate::Color32,
+    ) -> Arc<Galley> {
+        let job = LayoutJob::simple(text, text_style, color, f32::INFINITY);
+        self.layout_job(job)
+    }
+
+    /// Like [`Self::layout`], made for when you want to pick a color for the text later.
+    ///
+    /// The implementation uses memoization so repeated calls are cheap.
+    pub fn layout_delayed_color(
+        &self,
+        text: String,
+        text_style: TextStyle,
+        wrap_width: f32,
+    ) -> Arc<Galley> {
+        self.layout_job(LayoutJob::simple(
             text,
-            0.0,
-            max_width_in_points,
-        )
-    }
-
-    /// * `first_row_indentation`: extra space before the very first character (in points).
-    /// * `max_width_in_points`: wrapping width.
-    /// Always returns at least one row.
-    pub fn layout_multiline_with_indentation_and_max_width(
-        &self,
-        text_style: TextStyle,
-        text: String,
-        first_row_indentation: f32,
-        max_width_in_points: f32,
-    ) -> Arc<Galley> {
-        self.galley_cache.lock().layout(
-            &self.fonts,
-            LayoutJob {
-                text_style,
-                text,
-                layout_params: LayoutParams::Multiline {
-                    first_row_indentation: first_row_indentation.into(),
-                    max_width_in_points: max_width_in_points.into(),
-                },
-            },
-        )
+            text_style,
+            crate::Color32::TEMPORARY_COLOR,
+            wrap_width,
+        ))
     }
 
     pub fn num_galleys_in_cache(&self) -> usize {
@@ -352,7 +376,7 @@ impl Fonts {
 
     /// Must be called once per frame to clear the [`Galley`] cache.
     pub fn end_frame(&self) {
-        self.galley_cache.lock().end_frame()
+        self.galley_cache.lock().end_frame();
     }
 }
 
@@ -367,22 +391,6 @@ impl std::ops::Index<TextStyle> for Fonts {
 
 // ----------------------------------------------------------------------------
 
-#[derive(Clone, Copy, Eq, PartialEq, Hash)]
-enum LayoutParams {
-    SingleLine,
-    Multiline {
-        first_row_indentation: ordered_float::OrderedFloat<f32>,
-        max_width_in_points: ordered_float::OrderedFloat<f32>,
-    },
-}
-
-#[derive(Clone, Eq, PartialEq, Hash)]
-struct LayoutJob {
-    text_style: TextStyle,
-    layout_params: LayoutParams,
-    text: String,
-}
-
 struct CachedGalley {
     /// When it was last used
     last_used: u32,
@@ -393,41 +401,28 @@ struct CachedGalley {
 struct GalleyCache {
     /// Frame counter used to do garbage collection on the cache
     generation: u32,
-    cache: AHashMap<LayoutJob, CachedGalley>,
+    cache: nohash_hasher::IntMap<u64, CachedGalley>,
 }
 
 impl GalleyCache {
-    fn layout(&mut self, fonts: &BTreeMap<TextStyle, Font>, job: LayoutJob) -> Arc<Galley> {
-        if let Some(cached) = self.cache.get_mut(&job) {
-            cached.last_used = self.generation;
-            cached.galley.clone()
-        } else {
-            let LayoutJob {
-                text_style,
-                layout_params,
-                text,
-            } = job.clone();
-            let font = &fonts[&text_style];
-            let galley = match layout_params {
-                LayoutParams::SingleLine => font.layout_single_line(text),
-                LayoutParams::Multiline {
-                    first_row_indentation,
-                    max_width_in_points,
-                } => font.layout_multiline_with_indentation_and_max_width(
-                    text,
-                    first_row_indentation.into_inner(),
-                    max_width_in_points.into_inner(),
-                ),
-            };
-            let galley = Arc::new(galley);
-            self.cache.insert(
-                job,
-                CachedGalley {
+    fn layout(&mut self, fonts: &Fonts, job: LayoutJob) -> Arc<Galley> {
+        let hash = crate::util::hash(&job); // TODO: even faster hasher?
+
+        match self.cache.entry(hash) {
+            std::collections::hash_map::Entry::Occupied(entry) => {
+                let cached = entry.into_mut();
+                cached.last_used = self.generation;
+                cached.galley.clone()
+            }
+            std::collections::hash_map::Entry::Vacant(entry) => {
+                let galley = super::layout(fonts, job.into());
+                let galley = Arc::new(galley);
+                entry.insert(CachedGalley {
                     last_used: self.generation,
                     galley: galley.clone(),
-                },
-            );
-            galley
+                });
+                galley
+            }
         }
     }
 
@@ -450,7 +445,7 @@ impl GalleyCache {
 struct FontImplCache {
     atlas: Arc<Mutex<TextureAtlas>>,
     pixels_per_point: f32,
-    rusttype_fonts: BTreeMap<String, Arc<rusttype::Font<'static>>>,
+    ab_glyph_fonts: BTreeMap<String, ab_glyph::FontArc>,
 
     /// Map font names and size to the cached `FontImpl`.
     /// Can't have f32 in a HashMap or BTreeMap, so let's do a linear search
@@ -463,27 +458,22 @@ impl FontImplCache {
         pixels_per_point: f32,
         definitions: &super::FontDefinitions,
     ) -> Self {
-        let rusttype_fonts = definitions
+        let ab_glyph_fonts = definitions
             .font_data
             .iter()
-            .map(|(name, font_data)| {
-                (
-                    name.clone(),
-                    Arc::new(rusttype_font_from_font_data(name, font_data)),
-                )
-            })
+            .map(|(name, font_data)| (name.clone(), ab_glyph_font_from_font_data(name, font_data)))
             .collect();
 
         Self {
             atlas,
             pixels_per_point,
-            rusttype_fonts,
+            ab_glyph_fonts,
             cache: Default::default(),
         }
     }
 
-    pub fn rusttype_font(&self, font_name: &str) -> Arc<rusttype::Font<'static>> {
-        self.rusttype_fonts
+    pub fn ab_glyph_font(&self, font_name: &str) -> ab_glyph::FontArc {
+        self.ab_glyph_fonts
             .get(font_name)
             .unwrap_or_else(|| panic!("No font data found for {:?}", font_name))
             .clone()
@@ -512,7 +502,7 @@ impl FontImplCache {
         let font_impl = Arc::new(FontImpl::new(
             self.atlas.clone(),
             self.pixels_per_point,
-            self.rusttype_font(font_name),
+            self.ab_glyph_font(font_name),
             scale_in_points,
             y_offset,
         ));
