@@ -3,14 +3,16 @@
 pub mod cursor;
 mod font;
 mod fonts;
-mod galley;
+mod text_layout;
+mod text_layout_types;
 
 /// One `\t` character is this many spaces wide.
 pub const TAB_SIZE: usize = 4;
 
 pub use {
     fonts::{FontDefinitions, FontFamily, Fonts, TextStyle},
-    galley::{Galley, Row},
+    text_layout::layout,
+    text_layout_types::*,
 };
 
 /// Suggested character to use to replace those in password text fields.
